#!/usr/bin/env python3


import graphmbar as gmbar


class MySetup(gmbar.Analyzer):

    def __init__(self,args):
        super().__init__(args.expt,args.archive,args.bar,args.trials)
        self.SetRefMol(args.lead)
        self.SetExptEdgeConstraints(args.exptcon)
        

    # ######################################################################
    #
    # These routines are specific to the directory structure of this project
    #
    # ######################################################################


    def getdir( self, transform, env, stage, trial ):
        """
        transform -- string, name of the A->B transformation
        env -- string, the environment. This is either: "solvated" or "complex"
        stage -- string, the transformation step; e.g., "recharge", 
                 "decharge", "vdw"
        trial -- string, the independent trial
        
        returns a directory name where the efep_*_*.dat files are located
        """
    
#        return "dats/%s/free_energy/%s_ambest/%s/%s"%(trial,transform,env,stage)
        return "data/%s/%s/%i/"%(transform,env,int(trial))

    def getedges( self, arcfiles ):
        """
        arcfiles -- dict, keys are filenames and values are file sizes
                If an archive was not read, then this dict is empty
                and the files should be examined on the OS filesystem
    
        returns a list (strings) of transformation names using the format: A~B
        for the transformation A->B
        """
    
        import glob
        import os.path
        import re

        edges=[]

        if len([fname for fname in arcfiles]) > 0:
            #
            # Look at files in archive
            #
            p = re.compile(r"dats/1/free_energy/(.*)_ambest/.*")
            for fname in arcfiles:
                m = p.match(fname)
                if m:
                    if m.group(1) not in edges:
                        edges.append(m.group(1))
        else:
            #
            # Look at files on OS filesystem
            #
            fs = glob.glob( "data/*~*" )
            #fs = glob.glob( "dats/1/free_energy/*_ambest" )
            for f in fs:
                cs = f.split("/")
                c = cs[-1]
                c = c.replace("-","~")
                #c = c.replace("_ambest","")
                edges.append(c)
        edges.sort()
        return edges


    def getstagedlambdas(self):
        """
        returns a dict. The dict value is a list. The list elements are
        strings. The strings are values/labels for the lambda values used
        for the transformation. The keys you choose here are the "stage"
        values in getdir(), above
        """
        lams = {}
        lams["unified"] = [ "0.00000000","0.10000000","0.20000000",
                         "0.30000000","0.40000000","0.50000000",
                         "0.60000000","0.70000000","0.80000000",
                         "0.90000000","1.00000000" ]

        #lams["decharge"] = [ "0.00000000", "0.25000000", "0.50000000",
        #                     "0.75000000", "1.00000000" ]
        
        #lams["recharge"] = lams["decharge"]
        
        #lams["vdw"] = [ "0.00000000", "0.04790000", "0.11510000",
        #                "0.20630000", "0.31610000", "0.43740000",
        #                "0.56260000", "0.68390000", "0.79370000",
        #                "0.88500000", "0.95210000", "1.00000000" ]
        return lams



# ######################################################################
#
# The rest of the file should work for any directory structure
#
# ######################################################################



if __name__ == "__main__":
    
    import os
    import sys
    import argparse
    import pickle
    from collections import OrderedDict as odict
    from collections import defaultdict as ddict

    parser = argparse.ArgumentParser \
        ( formatter_class=argparse.RawDescriptionHelpFormatter,
          description="""
graphmbar input file generator and output file analyzer
""" )

    parser.add_argument \
        ("-f","--expt",
         help="Filename containing experimental dG values",
         type=str,
         required=True )

    parser.add_argument \
        ("-x","--exptcon",
         help="If present, then apply experimental ddG constraint between the two specified molecule names. Each instance of --exptcon takes two arguments.",
         nargs=2,
         type=str,
         action='append' )

    parser.add_argument \
        ("-l","--lead",
         help="string, name of lead ligand. If empty "", then the molecule with the most negative experimental free energy is chosen as the lead ligand.",
         type=str,
         default="",
         required=False )
    
    parser.add_argument \
        ("-a","--analyze",
         help="If present, then analyze the output file, otherwise, write an input file",
         type=str,
         default="",
         required=False)

    parser.add_argument \
        ("-t","--trials",
         help="a list of trials (strings) to analyze. Default: [\"1\",\"2\",\"3\",\"4\",\"5\",\"6\",\"7\",\"8\",\"9\",\"10\"]",
         type=str,
         nargs='+',
         default=["1","2","3","4","5","6","7","8","9","10"],
         required=False)

    parser.add_argument \
        ("-b","--bar",
         help="If present, then only check for files necessary for BAR calculations, otherwise make sure all files necessary for MBAR are available",
         default=False,
         action='store_true',
         required=False)

    parser.add_argument \
        ("-z","--archive",
         help="If present, then read files from the compressed archive",
         type=str,
         default="",
         required=False)

    #################################
    
    parser.add_argument \
        ("-r","--rescyc",
         help="If present, then explicitly restraint minimum-length cycle constraints",
         action='store_true',
         required=False )
    
    parser.add_argument \
        ("-c","--concyc",
         help="If present, then implicitly constrain all cycles using slack variables",
         action='store_true',
         required=False )
    
    parser.add_argument \
        ("-C","--noprintcycles",
         help="If present, then don't print cycles in the analysis",
         action='store_true',
         required=False )
 
    parser.add_argument \
        ("-P","--noprintpaths",
         help="If present, then don't print path analysis",
         action='store_true',
         required=False )

    
    args = parser.parse_args()

    ana = MySetup(args)

    if args.analyze:
        ana.AnalyzeOutput( args.analyze, not args.noprintcycles, not args.noprintpaths )
    else:
        ana.WriteInput(sys.stdout,args.rescyc,args.concyc)
    
